! AUTHOR: C. Guirao
! Modifications for FIES by A. Irrgang
!
! NAME: fies_pipeline.prg
!   Standard reduction MIDAS procedure for FIES echelle spectra
!
! INPUT: 
!   - CCD image in FITS format with science spectrum (e.g. sirius.fits)
!   - CCD image of master_bias in FITS format
!   - CCD image of master_flat in FITS format
!   - Slit width for extraction. Default 5
!
! OUTPUT: 
!   One-dimensional image (e.g. sirius_bsfwrm.fit) completely reduced
!   
! SUFFIX CONVENTION:
!   b with bias subtraction
!   s corrected for background, i.e., inter-order scattering
!   f with flat-field division
!   w calibrated in wavelength
!   r rebined
!   m orders merged
!
! USAGE: @@ fies_pipeline.prg frame.fit
!
!*******************************************************************************
! ECHO/FULL
!
! INPUT PARAMS:
! p1 REQUIRED: FITS image of spectrum to be reduced
! p2 REQUIRED: FITS image of master_bias FITS image
! p3 REQUIRED: FITS image of master_flat FITS image
! p4 OPTIONAL: Slit width for extraction in pixels
!
!
! Edit here to change defaults:
!
 DEFINE/PARA p1 ? 	  IMA "Enter FITS image to be calibrated: "
 DEFINE/PARA p2 ? 	  IMA "Enter master_bias FITS image: "
 DEFINE/PARA p3 ? 	  IMA "Enter master_flat FITS image: "
 DEFINE/PARA p4 5         NUM
!
! Convert paramaters into variables:
!
 DEFINE/LOCA cmd/c/1/20		"pipeline"
 DEFINE/LOCA fitsframe/c/1/40   {p1}
 DEFINE/LOCA fitsbias/c/1/40    {p2}
 DEFINE/LOCA fitsflat/c/1/40    {p3}
 DEFINE/LOCA width/i/1/1        {p4}
 DEFINE/LOCA original/c/1/40
 DEFINE/LOCA frame/c/1/40
 DEFINE/LOCA exposure/i/1/1	0

 DEFINE/LOCA bias/c/1/40
 DEFINE/LOCA flat/c/1/40
 DEFINE/LOCA backgr/c/1/40       "background.bdf"
 DEFINE/LOCA session/c/1/40      "FIES"
 DEFINE/LOCA printer/c/1/40      "postscript"
 DEFINE/LOCA session_file1/c/1/40
 DEFINE/LOCA session_file2/c/1/40
 DEFINE/LOCA tmpname/c/1/40
 DEFINE/LOCA iteration/i/1/1          1
 DEFINE/LOCA iterationplus/i/1/1   1
 DEFINE/LOCA start/d/1/1
 DEFINE/LOCA ende/d/1/1
 DEFINE/LOCA npix/i/1/1
 DEFINE/LOCA step/d/1/1
 DEFINE/LOCA ave/d/1/1
!
 DEFINE/LOCA reply/c/1/1        y
 DEFINE/LOCA fcentral/r/1/1	0
 DEFINE/LOCA fcentrum/i/1/1	0
 DEFINE/LOCA stat_flux/r/1/1    0.0
 DEFINE/LOCA basename/c/1/40    "" ? +lower_levels
 DEFINE/LOCA ext/c/1/10         "" ? +lower_levels
 DEFINE/LOCA next/i/1/1		1
!
! MAIN, main:
!
! Ask before any remove
!
 WRITE/OUT
 WRITE/OUT "{cmd}: Version: fies_pipeline"
 WRITE/OUT "{cmd}: Spectrum={fitsframe}"
 WRITE/OUT "{cmd}: Bias={fitsbias}"
 WRITE/OUT "{cmd}: Flat={fitsflat}"
 WRITE/OUT "{cmd}: Echelle session={session}"
 WRITE/OUT 
!
! Set context BACHES, if not yet already done
!
 SHOW/CONT baches >NULL
 IF  OUTPUTI(5) .eq. 0 THEN
   SET/CONTEXT baches >NULL
 ENDIF
!
! Set context ECHELLE, if not yet already done
!
 SHOW/CONT echelle > NULL
 IF  OUTPUTI(5) .eq. 0 THEN
   SET/CONTEXT echelle >NULL
 ENDIF
!
! Gets basename and extension of input file
!
 FILE/BACHES {fitsframe}
 original = basename
 frame = original + "_"
!
! Convert FITS format into MIDAS BDF format
!
 IF ext .ne. ".bdf" THEN
   COMPUTE/BACHES {fitsframe} {frame}
 ELSE
  $cp {original}{ext} {frame}{ext}
 ENDIF
!
! Gets basename and extension of input bias file
!
IF m$exist(fitsbias) .eq. 1 THEN
  FILE/BACHES {fitsbias}
  original = basename
  bias = original
!
! Convert FITS format into MIDAS BDF format
!
  IF ext .ne. ".bdf" THEN
    COMPUTE/BACHES {fitsbias} {bias}
  ELSE
    $cp {original}{ext} {bias}{ext}
  ENDIF
ELSE
   WRITE/OUT "{cmd}: {fitsbias} not found. No bias subtracted"
ENDIF
!
! Gets basename and extension of input flat file
!
IF m$exist(fitsflat) .eq. 1 THEN
  FILE/BACHES {fitsflat}
  original = basename
  flat = original + "_"
!
! Convert FITS format into MIDAS BDF format
!
  IF ext .ne. ".bdf" THEN
    COMPUTE/BACHES {fitsflat} {flat}
  ELSE
   $cp {original}{ext} {flat}{ext}
  ENDIF
ELSE
   WRITE/OUT "{cmd}: {fitsflat} not found. No flatfielding applied"
ENDIF
!
! Bias subtraction
!
 IF m$exist(bias+".bdf") .eq. 1 THEN 
   COMPUTE/IMA {frame}b = {frame} - {bias}
   frame = frame + "b"
   WRITE/OUT "{cmd}: creating {frame} with {bias} subtracted"
   IF m$exist(flat+".bdf") .eq. 1 THEN
      COMPUTE/IMA {flat}b = {flat} - {bias}
      flat = flat + "b"
      WRITE/OUT "{cmd}: creating {flat} with {bias} subtracted"
   ENDIF
 ELSE
   WRITE/OUT "{cmd}: {bias} not found. No bias subtracted"
 ENDIF
!
! Wavelength calibration 
!
 session_file1 = session + "ORDE.tbl"
 IF m$exist(session_file1) .ne. 1 THEN
   session_file1 = session + "ORDE.fit"
   tmpname = session + "ORDE"
   IF m$exist(session_file1) .eq. 1 THEN
     INDISK/FITS {session_file1} {tmpname}
   ENDIF
 ENDIF

 session_file2 = session + "LINE.tbl"
 IF m$exist(session_file2) .ne. 1 THEN
   session_file2 = session + "LINE.fit"
   tmpname = session + "LINE"
   IF m$exist(session_file2) .eq. 1 THEN
     INDISK/FITS {session_file2} {tmpname}
   ENDIF
 ENDIF

 session_file1 = session + "ORDE.tbl"
 session_file2 = session + "LINE.tbl"
 IF m$exist(session_file1) .eq. 1 THEN
   IF m$exist(session_file2) .eq. 1 THEN
!    ECHO/FULL
     INITIA/ECHE {session}
!
!    Background and scattering substraction:
!
!    SELECT/BACKGROUND
!    clear/chan over
     BACKGROUND/ECHELLE {frame} {backgr}  6,2,8 3 0 SPLINE
     SUBTRACT/BACKGROUND {frame} {backgr} {frame}s
     frame = frame + "s"
!
! Flat-fielding
!
    IF m$exist(flat+".bdf") .eq. 1 THEN
!
    BACKGROUND/ECHELLE {flat} {backgr} 6,2,8 3 0 SPLINE
    SUBTRACT/BACKGROUND {flat} {backgr} {flat}s
    flat = flat + "s"
    COMPUTE/IMA {frame}f = {frame} / {flat} >NULL
    frame = frame + "f"
    WRITE/OUT "{cmd}: creating {frame} with {flat} divided"
    ENDIF
!   
     SET/ECHE slit = {width}
     EXTRACT/ECHE {frame} {frame}w
     frame = frame + "w"
     WRITE/OUT "{cmd}: {frame} calibrated in wavelength"
     REBIN/ECHE {frame} {frame}r
     frame = frame + "r"
     WRITE/OUT "{cmd}: {frame} rebined"
     WRITE/OUT "{cmd}: converting {frame}.bdf to {frame}.fit"
!     OUTDISK/FITS {frame} {frame}.fit >NULL
     MERGE/ECHE {frame} orde 1,77 noappend
   ELSE
     WRITE/OUT "{cmd}: ERROR {session} incomplete:"
     WRITE/OUT "{cmd}: {session_file2} does not exist."
     RETURN/EXIT
   ENDIF
 ELSE
   WRITE/OUT "{cmd}: ERROR {session} incomplete:"
   WRITE/OUT "{cmd}: {session_file1} does not exist."
   RETURN/EXIT
 ENDIF
!
! Converting result into FITS
!
$ rename s/orde/{frame}/ orde????.bdf
 WRITE/OUT "{cmd}: Rescaling orders ..."
 iteration = 1
 begin_loop1:
  iterationplus = {iteration} + 1
  COPY/DKEY {frame}{iteration} START start
  COPY/DKEY {frame}{iteration} NPIX npix
  COPY/DKEY {frame}{iteration} STEP step
  ende = start + (npix-5)*step 
  COPY/DKEY {frame}{iterationplus} START start
  COPY/DKEY {frame}{iterationplus} STEP step
  start = start + 5*step
  STATIS/IMAG {frame}{iteration} [{start}:{ende}]  >NULL
  COMPUTE/KEYWORD ave = {outputr(3)}
  STATIS/IMAG {frame}{iterationplus} [{start}:{ende}]  >NULL
  COMPUT/IMAG temp = {frame}{iterationplus}*{ave}/{outputr(3)}
  COMPUT/IMAG {frame}{iterationplus} = temp
  IF iteration .lt. 75 THEN
   iteration = {iteration} + 1
   GOTO begin_loop1
  ENDIF

 WRITE/OUT "{cmd}: Merging orders ..."
 iteration = 1
 iterationplus = {iteration} + 1
 MERGE/SPECTRUM {frame}{iteration} {frame}{iterationplus} {frame}m{iterationplus} 5 RAMP >NULL
 iteration = 2
 begin_loop2:
  iterationplus = {iteration} + 1
  copy/dd {frame}m{iteration} STEP/D/01/1     {frame}{iterationplus} STEP/D/1/1
  MERGE/SPECTRUM {frame}m{iteration} {frame}{iterationplus} {frame}m{iterationplus} 5 RAMP >NULL
  IF iteration .lt. 75 THEN
   iteration = {iteration} + 1
   GOTO begin_loop2
  ENDIF
 COMPUT/IMAG {frame}m = {frame}m{iterationplus}
 frame = frame + "m"
!
 WRITE/OUT "{cmd}: converting {frame}.bdf to {frame}.fit"
  COMP/IMA &temp = {frame}         			! y-Plotranges automatisch angepasst
  COMP/IMA {frame} = &temp          			! y-Plotranges automatisch angepasst
  REPLAC/IMAG {frame} {frame} <,0=0		 	! keine negativen Pixel
  OUTDISK/FITS {frame} {frame}.fit >NULL
!
! Clean all temporary files.
!
CLEAN/BACHES
!
 ECHO/OFF